/*
 * $Log: gen_distort2_lookup.c,v $
 * Revision 1.4  2003/04/17 12:22:28  fonin
 * Strip lookup table before write it to the file.
 *
 * Revision 1.3  2003/04/16 13:56:21  fonin
 * Changed lookup table size to 64k.
 *
 * Revision 1.2  2003/04/12 20:03:00  fonin
 * Minor bugfix in the algorithm, and few variable type changes.
 *
 * Revision 1.1  2003/04/11 18:33:00  fonin
 * Lookup table generator for distortion2.
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "pump.h"

#define MAX_TUBE MAX_SAMPLE
#define FILL 32767
SAMPLE          tube[MAX_SAMPLE];

void
init_distort2_lookup(char *c_r1, char *c_r2, char *sr)
{
    int             sample_rate;
    int             r1 = 1;
    int             r2 = 510;
    int             res;
    float           c = 0.041;
    int             is = 10;
    long            mut = 2000000;	// 500000
    float           a1,
                    a2;
    float           z = 0.0000239;
    float           s2;
    int             i = 0,
                    j;
    float           x;
    float           x2,
                    y,
                    rz,
                    rz2;
    char            filename[255] = "";

    r1 = atoi(c_r1);
    r2 = atoi(c_r2);
    sample_rate = atoi(sr);

    printf
	("\nInitializing distortion2 lookup table: r1=%i, r2=%i, sampling rate=%i\n",
	 r1, r2, sample_rate);
    memset(tube, 0, sizeof(tube));

    /*
     * create lookup table 
     */
    a1 = (r1 + r2) * c;
    a2 = r1 * c;

    s2 = 2 * sample_rate * (z - 1) / (z + 1);
    for (i = 1; i <= MAX_TUBE; i++) {
	x = i;
	x2 = x * (s2 * a1 + 1) / (s2 * a2 + 1);
	y = 1;
	rz = ((x2 - y) / r2) + is * (exp((x - y) / mut) -
				     exp((y - x) / mut)) * 1000;
	tube[i] = i;
	for (j = 2; j <= MAX_TUBE; j++) {
	    y = j;
	    rz2 =
		((x2 - y) / r2) + is * (exp((x - y) / mut) -
					exp((y - x) / mut)) * 1000;
	    if (rz2 > 0 && rz2 < rz) {
		rz = rz2;
		tube[i] = j;
	    }
	}
	if (tube[i] > MAX_SAMPLE)
	    tube[i] = MAX_SAMPLE;
	if (i % 30 == 0) {
	    printf(".");
	}
    }

    /*
     * write the table to the file 
     */
    strcpy(filename, "distort2lookup_");
    strncat(filename, sr, 255);
    strcat(filename, "_");
    strncat(filename, c_r1, 255);
    strcat(filename, "_");
    strncat(filename, c_r2, 255);
    res =
	open(filename, O_CREAT | O_WRONLY | O_TRUNC,
	     S_IREAD | S_IWRITE | S_IRGRP | S_IROTH);
    if (res == -1) {
	perror("open");
	exit(1);
    }
    for(i=MAX_SAMPLE-1;i>=0;i--) {
	if(tube[i]!=FILL)
	    break;
    }

    write(res,tube,(i+1)*sizeof(SAMPLE));
    close(res);
}

int
main(int argc, char **argv)
{
    time_t          t1,
                    t2;
    if (argc != 4) {
	printf("\nLookup table generator for distortion 2 effect."
	       "\nUsage: %s r1 r2 sample_rate\n", argv[0]);
	exit(0);
    }

    t1 = time(NULL);

    /*
     * unbuffered output 
     */
    setvbuf(stdout, NULL, _IONBF, 0);

    memset(tube, 0, sizeof(tube));
    init_distort2_lookup(argv[1], argv[2], argv[3]);

    t2 = time(NULL);
    printf("\nComplete in %i seconds.\n", t2 - t1);
}
